@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id$
+/*  $Id: xfce-panel-convenience.c 24140 2006-12-19 19:36:18Z jasper $
  *
  *  Copyright © 2006 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -44,6 +44,8 @@ GtkWidget *xfce_create_panel_button (void)
     gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
     gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
 
+    gtk_widget_set_name (button, "xfce_panel_button");
+
     return button;
 }
 
@@ -63,6 +65,8 @@ GtkWidget *xfce_create_panel_toggle_button (void)
     gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
     gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
 
+    gtk_widget_set_name (button, "xfce_panel_toggle_button");
+
     return button;
 }
 
