@@ -1,8 +1,9 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id$
+/*  $Id: panel-dialogs.c 25157 2007-03-13 19:45:22Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
+ *  Copyright © 2006 Benedikt Meurer <benny@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published 
@@ -30,8 +31,10 @@
 #include <libxfcegui4/libxfcegui4.h>
 
 #include <libxfce4panel/xfce-itembar.h>
+#include <libxfce4panel/xfce-panel-macros.h>
 #include <libxfce4panel/xfce-panel-item-iface.h>
 
+#include "frap-icon-entry.h"
 #include "panel-properties.h"
 #include "panel-private.h"
 #include "panel-item-manager.h"
@@ -54,6 +57,7 @@ struct _PanelItemsDialog
     GtkWidget *active;
     
     GPtrArray *items;
+    GtkWidget *search_entry;
     GtkWidget *tree;
     GtkWidget *items_box;
 
@@ -147,7 +151,12 @@ add_selected_item (PanelItemsDialog *pid)
     GtkWidget *item = NULL;
 
     sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (pid->tree));
-    gtk_tree_selection_get_selected (sel, &model, &iter);
+    
+    if (!sel)
+        return FALSE;
+
+    if (!gtk_tree_selection_get_selected (sel, &model, &iter))
+        return FALSE;
 
     gtk_tree_model_get (model, &iter, 0, &info, -1);
 
@@ -196,9 +205,12 @@ cursor_changed (GtkTreeView * tv, PanelItemsDialog *pid)
     GtkTreeModel *model;
     GtkTreeIter iter;
     XfcePanelItemInfo *info;
-    
-    sel = gtk_tree_view_get_selection (tv);
-    gtk_tree_selection_get_selected (sel, &model, &iter);
+
+    if (!(sel = gtk_tree_view_get_selection (tv)))
+        return;
+
+    if (!gtk_tree_selection_get_selected (sel, &model, &iter))
+        return;
 
     gtk_tree_model_get (model, &iter, 0, &info, -1);
 }
@@ -208,7 +220,7 @@ treeview_destroyed (GtkWidget * tv)
 {
     GtkTreeModel *store;
 
-    store = gtk_tree_view_get_model (GTK_TREE_VIEW (tv));
+    store = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (gtk_tree_view_get_model (GTK_TREE_VIEW (tv))));
     gtk_list_store_clear (GTK_LIST_STORE (store));
 }
 
@@ -316,7 +328,8 @@ treeview_data_get (GtkWidget *widget, GdkDragContext *drag_context,
             return;
         }
         
-        gtk_tree_selection_get_selected (sel, &model, &iter);
+        if (!gtk_tree_selection_get_selected (sel, &model, &iter))
+            return;
 
         gtk_tree_model_get (model, &iter, 0, &info, -1);
 
@@ -329,6 +342,55 @@ treeview_data_get (GtkWidget *widget, GdkDragContext *drag_context,
     }
 }
 
+static gboolean
+item_visible_func (GtkTreeModel *model,
+                   GtkTreeIter  *iter,
+                   gpointer      user_data)
+{
+    XfcePanelItemInfo *info;
+    const gchar       *text;
+    GtkWidget         *entry = GTK_WIDGET (user_data);
+    gboolean           visible;
+    gchar             *text_casefolded;
+    gchar             *info_casefolded;
+    gchar             *normalized;
+
+    text = gtk_entry_get_text (GTK_ENTRY (entry));
+    if (G_UNLIKELY (*text == '\0'))
+        return TRUE;
+
+    gtk_tree_model_get (model, iter, 0, &info, -1);
+    if (G_UNLIKELY (info == NULL))
+        return TRUE;
+
+    normalized = g_utf8_normalize (text, -1, G_NORMALIZE_ALL);
+    text_casefolded = g_utf8_casefold (normalized, -1);
+    g_free (normalized);
+
+    normalized = g_utf8_normalize (info->display_name, -1, G_NORMALIZE_ALL);
+    info_casefolded = g_utf8_casefold (normalized, -1);
+    g_free (normalized);
+
+    visible = (strstr (info_casefolded, text_casefolded) != NULL);
+
+    g_free (info_casefolded);
+
+    if (!visible && info->comment)
+    {
+        normalized = g_utf8_normalize (info->comment, -1, G_NORMALIZE_ALL);
+        info_casefolded = g_utf8_casefold (normalized, -1);
+        g_free (normalized);
+
+        visible = (strstr (info_casefolded, text_casefolded) != NULL);
+
+        g_free (info_casefolded);
+    }
+
+    g_free (text_casefolded);
+
+    return visible;
+}
+
 static void
 add_item_treeview (PanelItemsDialog *pid)
 {
@@ -336,6 +398,7 @@ add_item_treeview (PanelItemsDialog *pid)
     GtkCellRenderer *cell;
     GtkTreeViewColumn *col;
     GtkListStore *store;
+    GtkTreeModel *filter;
     GtkTreeModel *model;
     GtkTreePath *path;
     GtkTreeIter iter;
@@ -354,7 +417,11 @@ add_item_treeview (PanelItemsDialog *pid)
     store = gtk_list_store_new (1, G_TYPE_POINTER);
     model = GTK_TREE_MODEL (store);
 
-    pid->tree = tv = gtk_tree_view_new_with_model (model);
+    filter = gtk_tree_model_filter_new (model, NULL);
+    gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter), item_visible_func, pid->search_entry, NULL);
+    g_signal_connect_swapped (G_OBJECT (pid->search_entry), "changed", G_CALLBACK (gtk_tree_model_filter_refilter), filter);
+
+    pid->tree = tv = gtk_tree_view_new_with_model (filter);
     gtk_widget_show (tv);
     gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tv), FALSE);
@@ -362,6 +429,7 @@ add_item_treeview (PanelItemsDialog *pid)
 
     g_signal_connect (tv, "destroy", G_CALLBACK (treeview_destroyed), NULL);
 
+    g_object_unref (G_OBJECT (filter));
     g_object_unref (G_OBJECT (store));
 
     /* dnd */
@@ -481,7 +549,7 @@ item_dialog_response (GtkWidget *dlg, int response, PanelItemsDialog *pid)
         gtk_widget_destroy (dlg);
         
         g_signal_handler_disconnect (pid->panel, pid->panel_destroy_id);
-        g_free (pid);
+        panel_slice_free (PanelItemsDialog, pid);
 
         panel_app_save ();
     }
@@ -512,7 +580,7 @@ add_items_dialog (GPtrArray *panels, GtkWidget *active_item)
         return;
     }
     
-    pid = g_new0 (PanelItemsDialog, 1);
+    pid = panel_slice_new0 (PanelItemsDialog);
 
     /* panels */
     pid->panels = panels;
@@ -552,40 +620,53 @@ add_items_dialog (GPtrArray *panels, GtkWidget *active_item)
 
     img = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, 
                                     GTK_ICON_SIZE_LARGE_TOOLBAR);
-    gtk_misc_set_alignment (GTK_MISC (img), 0, 0);
+    gtk_misc_set_alignment (GTK_MISC (img), 0.0f, 0.5f);
     gtk_widget_show (img);
     gtk_box_pack_start (GTK_BOX (hbox), img, FALSE, FALSE, 0);
 
     label = gtk_label_new (_("Drag items from the list to a panel or remove "
                              "them by dragging them back to the list."));
     gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
     gtk_widget_show (label);
     gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
-    
+
     /* treeview */
+    hbox = gtk_hbox_new (FALSE, BORDER);
+    gtk_widget_show (hbox);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+
     label = gtk_label_new (NULL);
-    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+    gtk_misc_set_alignment (GTK_MISC (label), 0, 1.0f);
     gtk_widget_show (label);
-    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
 
     markup = g_strdup_printf ("<b>%s</b>", _("Available Items"));
     gtk_label_set_markup (GTK_LABEL (label), markup);
     g_free (markup);
 
+    /* the list filter entry (FIXME: Add tooltip? Jasper?) */
+    pid->search_entry = frap_icon_entry_new ();
+    frap_icon_entry_set_stock_id (FRAP_ICON_ENTRY (pid->search_entry), GTK_STOCK_FIND);
+    gtk_widget_show (pid->search_entry);
+    gtk_box_pack_end (GTK_BOX (hbox), pid->search_entry, FALSE, FALSE, 0);
+    
     add_item_treeview (pid);
 
      /* make panels insensitive and set up dnd */
     g_ptr_array_foreach (panels, (GFunc)item_dialog_opened, NULL);
 
-    gtk_window_set_keep_above (GTK_WINDOW (dlg), TRUE);
     gtk_window_stick(GTK_WINDOW (dlg));
-    gtk_window_set_type_hint (GTK_WINDOW (dlg), GDK_WINDOW_TYPE_HINT_UTILITY);
 
+    gtk_widget_realize (dlg);
+    gdk_x11_window_set_user_time (GTK_WIDGET (dlg)->window,
+        gdk_x11_get_server_time (GTK_WIDGET (dlg)->window));
     xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dlg));
     gtk_widget_show (dlg);
 
     panel_app_register_dialog (dlg);
+
+    gtk_window_present (GTK_WINDOW (dlg));
 }
 
 /* 
@@ -1098,7 +1179,7 @@ add_position_options (GtkBox *box, PanelManagerDialog *pmd)
     g_signal_connect (pmd->handle_style, "changed", 
                       G_CALLBACK (handle_style_changed), pmd);
     
-    g_object_unref (sg);
+    g_object_unref (G_OBJECT (sg));
 }
 
 /* monitors */
@@ -1126,16 +1207,8 @@ monitor_pressed (GtkToggleButton *tb, GdkEvent *ev, PanelManagerDialog *pmd)
 
                 if (mon == tb)
                 {
-                    XfceMonitor *xmon = panel_app_get_monitor(i);
                     gtk_toggle_button_set_active (mon, TRUE);
                     panel_set_monitor (pmd->panel, i);
-                    if (xmon->screen != gtk_widget_get_screen (pmd->dlg))
-                    {
-                        gtk_widget_hide (pmd->dlg);
-                        gtk_window_set_screen (GTK_WINDOW (pmd->dlg), 
-                                               xmon->screen);
-                        gtk_widget_show (pmd->dlg);
-                    }
                 }
                 else
                 {
@@ -1307,8 +1380,13 @@ add_appearance_options (GtkBox *box, PanelManagerDialog *pmd)
 
     /* transparency */
     if (G_UNLIKELY (!composite_atom))
+    {
+        char text[16];
+        g_snprintf (text, 16, "_NET_WM_CM_S%d", 
+                    GDK_SCREEN_XNUMBER(gdk_screen_get_default()));
         composite_atom = 
-            XInternAtom (GDK_DISPLAY (), "COMPOSITING_MANAGER", False);
+            XInternAtom (GDK_DISPLAY (), text, False);
+    }
 
     if (XGetSelectionOwner (GDK_DISPLAY (), composite_atom))
     {
@@ -1490,8 +1568,8 @@ panel_dialog_response (GtkWidget *dlg, int response, PanelManagerDialog *pmd)
         
         gtk_widget_destroy (dlg);
         
-        g_object_unref (pmd->tips);
-        g_free (pmd);
+        g_object_unref (G_OBJECT (pmd->tips));
+        panel_slice_free (PanelManagerDialog, pmd);
 
         panel_app_save ();
     }
@@ -1515,7 +1593,7 @@ panel_manager_dialog (GPtrArray *panels)
         return;
     }
     
-    pmd = g_new0 (PanelManagerDialog, 1);
+    pmd = panel_slice_new0 (PanelManagerDialog);
 
     /* panels */
     pmd->panels = panels;
@@ -1534,7 +1612,7 @@ panel_manager_dialog (GPtrArray *panels)
     gtk_window_set_icon_name (GTK_WINDOW (pmd->dlg), "xfce4-panel");
 
     pmd->tips = gtk_tooltips_new ();
-    g_object_ref (pmd->tips);
+    g_object_ref (G_OBJECT (pmd->tips));
     gtk_object_sink (GTK_OBJECT (pmd->tips));
 
     /* main container */
@@ -1572,12 +1650,13 @@ panel_manager_dialog (GPtrArray *panels)
     g_signal_connect (pmd->dlg, "response", 
                       G_CALLBACK (panel_dialog_response), pmd);
     
-    gtk_window_set_keep_above (GTK_WINDOW (pmd->dlg), TRUE);
     gtk_window_stick(GTK_WINDOW (pmd->dlg));
-    gtk_window_set_type_hint (GTK_WINDOW (pmd->dlg), 
-                              GDK_WINDOW_TYPE_HINT_UTILITY);
-
     xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (pmd->dlg));
+    gtk_widget_realize (pmd->dlg);
+
+    /* window needs to be realized */
+    gdk_x11_window_set_user_time (GTK_WIDGET (pmd->dlg)->window,
+        gdk_x11_get_server_time (GTK_WIDGET (pmd->dlg)->window));
     gtk_widget_show (pmd->dlg);
 
     panel_app_register_dialog (pmd->dlg);
