@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel-properties.c 22660 2006-08-04 21:02:22Z jasper $
+/*  $Id: panel-properties.c 25300 2007-03-27 12:06:38Z nick $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -36,8 +36,9 @@
 
 #include "panel-properties.h"
 #include "panel-private.h"
+#include "panel-dnd.h"
 
-#define HIDDEN_SIZE	3
+#define HIDDEN_SIZE     3
 #define OPAQUE          0xffffffff
 #define HIDE_TIMEOUT    350
 #define UNHIDE_TIMEOUT  200
@@ -45,33 +46,37 @@
 
 /* automatic (re)sizing and moving */
 static void panel_move_end            (XfcePanelWindow *window,
-                                       int x,
-                                       int y);
+                                       int              x,
+                                       int              y);
 
 static void panel_move_function       (XfcePanelWindow *window,
-                                       Panel *panel,
-                                       int *x,
-                                       int *y);
+                                       Panel           *panel,
+                                       int             *x,
+                                       int             *y);
 
 static void panel_resize_function     (XfcePanelWindow *window,
-                                       Panel *panel,
-                                       GtkAllocation *old,
-                                       GtkAllocation *new,
-                                       int *x,
-                                       int *y);
+                                       Panel           *panel,
+                                       GtkAllocation   *old,
+                                       GtkAllocation   *new,
+                                       int             *x,
+                                       int             *y);
 
 /* autohide and transparency */
-static void panel_enter (Panel *p, GdkEventCrossing * event);
+static void panel_enter               (Panel            *p, 
+                                       GdkEventCrossing *event);
 
-static void panel_leave (Panel *p, GdkEventCrossing * event);
+static void panel_leave               (Panel            *p, 
+                                       GdkEventCrossing *event);
 
 
 /* moving and resizing */
 static void
-_calculate_coordinates (XfceScreenPosition position, /* screen position */
-                        GdkRectangle *screen,         /* monitor geometry */
-                        int width, int height,        /* panel size */
-                        int *x, int *y)               /* output coordinates */
+_calculate_coordinates (XfceScreenPosition  position, 
+                        GdkRectangle       *screen,   /* monitor geometry */
+                        int                 width, 
+                        int                 height,   
+                        int                *x, 
+                        int                *y)               
 {
     switch (position)
     {
@@ -84,54 +89,66 @@ _calculate_coordinates (XfceScreenPosition position, /* screen position */
             *y = CLAMP (*y, screen->y, 
                         MAX (screen->y, screen->y + screen->height - height));
             break;
+
         /* top */
         case XFCE_SCREEN_POSITION_NW_H:
             *x = screen->x;
             *y = screen->y;
             break;
+
         case XFCE_SCREEN_POSITION_N:
             *x = MAX (screen->x, screen->x + (screen->width - width) / 2);
             *y = screen->y;
             break;
+
         case XFCE_SCREEN_POSITION_NE_H:
             *x = MAX (screen->x, screen->x + screen->width - width);
             *y = screen->y;
             break;
+
         /* left */
         case XFCE_SCREEN_POSITION_NW_V:
             *x = screen->x;
             *y = screen->y;
             break;
+
         case XFCE_SCREEN_POSITION_W:
             *x = screen->x;
             *y = MAX (screen->y, screen->y + (screen->height - height) / 2);
             break;
+
         case XFCE_SCREEN_POSITION_SW_V:
             *x = screen->x;
             *y = MAX (screen->y, screen->y + screen->height - height);
             break;
+
         /* right */
         case XFCE_SCREEN_POSITION_NE_V:
             *x = screen->x + screen->width - width;
             *y = screen->y;
             break;
+
         case XFCE_SCREEN_POSITION_E:
             *x = screen->x + screen->width - width;
             *y = MAX (screen->y, screen->y + (screen->height - height) / 2);
             break;
+
         case XFCE_SCREEN_POSITION_SE_V:
             *x = screen->x + screen->width - width;
             *y = MAX (screen->y, screen->y + screen->height - height);
             break;
+
         /* bottom */
         case XFCE_SCREEN_POSITION_SW_H:
             *x = screen->x;
             *y = screen->y + screen->height - height;
             break;
+
         case XFCE_SCREEN_POSITION_S:
             *x = MAX (screen->x, screen->x + (screen->width - width) / 2);
             *y = screen->y + screen->height - height;
             break;
+
         case XFCE_SCREEN_POSITION_SE_H:
             *x = MAX (screen->x, screen->x + screen->width - width);
             *y = screen->y + screen->height - height;
@@ -143,7 +160,7 @@ static void
 _set_borders (Panel *panel)
 {
     PanelPrivate *priv;
-    gboolean top, bottom, left, right;
+    gboolean      top, bottom, left, right;
 
     priv = panel->priv;
 
@@ -156,69 +173,68 @@ _set_borders (Panel *panel)
         case XFCE_SCREEN_POSITION_FLOATING_H:
         case XFCE_SCREEN_POSITION_FLOATING_V:
             break;
+
         /* top */
         case XFCE_SCREEN_POSITION_NW_H:
             top = left = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                right = FALSE;
+            right = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         case XFCE_SCREEN_POSITION_N:
             top = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                left = right = FALSE;
+            left = right = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         case XFCE_SCREEN_POSITION_NE_H:
             top = right = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                left = FALSE;
+            left = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         /* left */
         case XFCE_SCREEN_POSITION_NW_V:
             left = top = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                bottom = FALSE;
+            bottom = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         case XFCE_SCREEN_POSITION_W:
             left = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                top = bottom = FALSE;
+            top = bottom = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         case XFCE_SCREEN_POSITION_SW_V:
             left = bottom = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                top = FALSE;
+            top = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         /* right */
         case XFCE_SCREEN_POSITION_NE_V:
             right = top = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                bottom = FALSE;
+            bottom = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         case XFCE_SCREEN_POSITION_E:
             right = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                top = bottom = FALSE;
+            top = bottom = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
         case XFCE_SCREEN_POSITION_SE_V:
             right = bottom = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                top = FALSE;
+            top = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         /* bottom */
         case XFCE_SCREEN_POSITION_SW_H:
             bottom = left = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                right = FALSE;
+            right = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         case XFCE_SCREEN_POSITION_S:
             bottom = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                left = right = FALSE;
+            left = right = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
+
         case XFCE_SCREEN_POSITION_SE_H:
             bottom = right = FALSE;
-            if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
-                left = FALSE;
+            left = (priv->full_width == XFCE_PANEL_NORMAL_WIDTH);
             break;
     }
 
@@ -227,10 +243,17 @@ _set_borders (Panel *panel)
 }
 
 static void
-_set_struts (Panel *panel, XfceMonitor *xmon, int x, int y, int w, int h)
+_set_struts (Panel       *panel, 
+             XfceMonitor *xmon, 
+             int          x, 
+             int          y, 
+             int          w, 
+             int          h)
 {
     PanelPrivate *priv;
-    gulong data[12] = { 0, };
+    gulong        data[12] = { 0, };
+    int           i;
+    gboolean      update = FALSE;
 
     /* _NET_WM_STRUT_PARTIAL: CARDINAL[12]/32
      * 
@@ -244,6 +267,10 @@ _set_struts (Panel *panel, XfceMonitor *xmon, int x, int y, int w, int h)
 
     priv = panel->priv;
 
+    /* use edit_mode property to test if we are changing position */
+    if (G_UNLIKELY (priv->edit_mode))
+        return;
+
     if (!priv->autohide && 
             !xfce_screen_position_is_floating (priv->screen_position))
     {
@@ -256,8 +283,8 @@ _set_struts (Panel *panel, XfceMonitor *xmon, int x, int y, int w, int h)
             {
                 data[0] = xmon->geometry.x 
                           + w;          /* left           */
-                data[4] = y;	        /* left_start_y   */
-                data[5] = y + h;	/* left_end_y     */
+                data[4] = y;            /* left_start_y   */
+                data[5] = y + h;        /* left_end_y     */
             }
         }
         else if (xfce_screen_position_is_right (priv->screen_position))
@@ -270,8 +297,8 @@ _set_struts (Panel *panel, XfceMonitor *xmon, int x, int y, int w, int h)
                 data[1] =  gdk_screen_get_width (xmon->screen) 
                            - xmon->geometry.x - xmon->geometry.width 
                            + w;         /* right          */
-                data[6] = y;	        /* right_start_y  */
-                data[7] = y + h;	/* right_end_y    */
+                data[6] = y;            /* right_start_y  */
+                data[7] = y + h;        /* right_end_y    */
             }
         }
         else if (xfce_screen_position_is_top (priv->screen_position))
@@ -282,9 +309,9 @@ _set_struts (Panel *panel, XfceMonitor *xmon, int x, int y, int w, int h)
             if (!xmon->has_neighbor_above)
             {
                 data[2] = xmon->geometry.y 
-                          + h;	        /* top            */
-                data[8] = x;	        /* top_start_x    */
-                data[9] = x + w;	/* top_end_x      */
+                          + h;          /* top            */
+                data[8] = x;            /* top_start_x    */
+                data[9] = x + w;        /* top_end_x      */
             }
         }
         else
@@ -296,43 +323,95 @@ _set_struts (Panel *panel, XfceMonitor *xmon, int x, int y, int w, int h)
             {
                 data[3] = gdk_screen_get_height (xmon->screen) 
                            - xmon->geometry.y - xmon->geometry.height 
-                           + h;	        /* bottom         */
-                data[10] = x;	        /* bottom_start_x */
-                data[11] = x + w;	/* bottom_end_x   */
+                           + h;         /* bottom         */
+                data[10] = x;           /* bottom_start_x */
+                data[11] = x + w;       /* bottom_end_x   */
             }
-	}
+        }
     }
 
-    DBG ("\nStruts: "
-	 "%ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld",
-	 data[0], data[1], data[2], data[3], data[4], data[5],
-	 data[6], data[7], data[8], data[9], data[10], data[11]);
+    DBG ("\nStruts\n"
+         "%ld\t%ld\t%ld\t%ld\n"
+         "%ld\t%ld\t%ld\t%ld\n"
+         "%ld\t%ld\t%ld\t%ld\n",
+         data[0], data[1], data[2], data[3], 
+         data[4], data[5], data[6], data[7], 
+         data[8], data[9], data[10], data[11]);
 
-    gdk_error_trap_push ();
+    /* Check for invalid values. */
+    for (i = 0; i < 4; i++)
+    {
+        if (data[i] > MIN(xmon->geometry.width,xmon->geometry.height) / 4)
+        {
+            return;
+        }
+    }
+    
+    /* Check if values have changed. */
+    for (i = 0; i < 12; i++)
+    {
+        if (data[i] != priv->struts[i])
+        {
+            update = TRUE;
+            priv->struts[i] = data[i];
+        }
+    }
+    
+    if (update)
+    {
+        gdk_error_trap_push ();
 
-    gdk_property_change (GTK_WIDGET (panel)->window,
-			 gdk_atom_intern ("_NET_WM_STRUT_PARTIAL", FALSE),
-			 gdk_atom_intern ("CARDINAL", FALSE), 32,
-			 GDK_PROP_MODE_REPLACE, (guchar *) & data, 12);
+        gdk_property_change (GTK_WIDGET (panel)->window,
+                             gdk_atom_intern ("_NET_WM_STRUT_PARTIAL", FALSE),
+                             gdk_atom_intern ("CARDINAL", FALSE), 32,
+                             GDK_PROP_MODE_REPLACE, (guchar *) & data, 12);
 
-    gdk_property_change (GTK_WIDGET (panel)->window,
-			 gdk_atom_intern ("_NET_WM_STRUT", FALSE),
-			 gdk_atom_intern ("CARDINAL", FALSE), 32,
-			 GDK_PROP_MODE_REPLACE, (guchar *) & data, 4);
+        gdk_property_change (GTK_WIDGET (panel)->window,
+                             gdk_atom_intern ("_NET_WM_STRUT", FALSE),
+                             gdk_atom_intern ("CARDINAL", FALSE), 32,
+                             GDK_PROP_MODE_REPLACE, (guchar *) & data, 4);
 
-    gdk_error_trap_pop ();
+        gdk_error_trap_pop ();
+    }
+    
+    DBG ("all struts are checked and updated");
+}
+
+static gboolean
+unblock_struts (Panel *panel)
+{
+    PanelPrivate *priv;
+    XfceMonitor  *xmon;
+    int           x, y, w, h;
+
+    priv = PANEL (panel)->priv;
+    priv->edit_mode = FALSE;
+
+    gtk_window_get_position (GTK_WINDOW (panel), &x, &y);
+    gtk_window_get_size (GTK_WINDOW (panel), &w, &h);
+
+    xmon = panel_app_get_monitor (priv->monitor);
+
+    _set_struts (panel, xmon, x, y, w, h);
+    return FALSE;
 }
 
 static void 
-panel_move_end (XfcePanelWindow *window, int x, int y)
+panel_move_end (XfcePanelWindow *window, 
+                int              x, 
+                int              y)
 {
     PanelPrivate *priv;
-    XfceMonitor *xmon;
+    XfceMonitor  *xmon;
 
     priv = PANEL (window)->priv;
 
     if (xfce_screen_position_is_floating (priv->screen_position))
     {
+        /* Emit this signal to allow plugins to do something based on the
+         * new position. E.g. the launcher plugin adjusts the arrow type
+         * of the associated menu button.
+         */
         gtk_container_foreach (GTK_CONTAINER (priv->itembar),
                 (GtkCallback)xfce_panel_item_set_screen_position,
                 GINT_TO_POINTER (priv->screen_position));
@@ -350,107 +429,151 @@ panel_move_end (XfcePanelWindow *window, int x, int y)
 }
 
 static void 
-panel_move_function (XfcePanelWindow *window, Panel *panel, int *x, int *y)
+panel_move_function (XfcePanelWindow *window, 
+                     Panel           *panel, 
+                     int             *x, 
+                     int             *y)
 {
     PanelPrivate *priv;
-    XfceMonitor *xmon;
+    GdkScreen    *screen;
+    XfceMonitor  *xmon;
 
     priv = panel->priv;
-
-    priv->monitor = gdk_screen_get_monitor_at_point (
-                        gtk_widget_get_screen (GTK_WIDGET (window)), *x, *y);
+    screen = gtk_widget_get_screen (GTK_WIDGET (window));
+    priv->monitor = gdk_screen_get_monitor_at_point (screen, *x, *y);
     
     DBG (" + Monitor at (%d, %d) = %d\n", *x, *y, priv->monitor);
+
     xmon = panel_app_get_monitor (priv->monitor);
     
-    _calculate_coordinates (priv->screen_position, &(xmon->geometry),
+    _calculate_coordinates (priv->screen_position, 
+                            &(xmon->geometry),
                             GTK_WIDGET (window)->allocation.width, 
                             GTK_WIDGET (window)->allocation.height, 
                             x, y);
 }
 
 static void 
-panel_resize_function (XfcePanelWindow *window, Panel *panel, 
-                       GtkAllocation *old, GtkAllocation *new, int *x, int *y)
+panel_resize_function (XfcePanelWindow *window, 
+                       Panel           *panel, 
+                       GtkAllocation   *old, 
+                       GtkAllocation   *new, 
+                       int             *x, 
+                       int             *y)
 {
     PanelPrivate *priv;
-    XfceMonitor *xmon;
+    XfceMonitor  *xmon;
+    GdkRectangle  rect;
+
+    DBG ("old: %dx%d\tnew: %dx%d", 
+         old ? old->width : 0, old ? old->height : 0,
+         new->width, new->height );
 
     if (!GTK_WIDGET_VISIBLE (panel))
         return;
-    
-    priv = panel->priv;
 
+    priv = panel->priv;
     xmon = panel_app_get_monitor (priv->monitor);
     
-    _calculate_coordinates (priv->screen_position, &(xmon->geometry),
-                            new->width, new->height, x, y);
+    if (priv->full_width < XFCE_PANEL_SPAN_MONITORS)
+    {
+        _calculate_coordinates (priv->screen_position, 
+                                &(xmon->geometry),
+                                new->width, 
+                                new->height, 
+                                x, y);
     
-    priv->xoffset = *x - xmon->geometry.x;
-    priv->yoffset = *y - xmon->geometry.y;
-    
-    DBG ("\n + Position: %d\n + Offset: (%d, %d)", 
-         priv->screen_position, priv->xoffset, priv->yoffset);
+        priv->xoffset = *x - xmon->geometry.x;
+        priv->yoffset = *y - xmon->geometry.y;
+    }
+    else
+    {
+        rect.x      = 0;
+        rect.y      = 0;
+        rect.width  = gdk_screen_get_width (xmon->screen);
+        rect.height = gdk_screen_get_height (xmon->screen);
 
+        _calculate_coordinates (priv->screen_position, 
+                                &rect,
+                                new->width, 
+                                new->height, 
+                                x, y);
+    
+        priv->xoffset = *x;
+        priv->yoffset = *y;
+    }
+    
     _set_struts (panel, xmon, *x, *y, new->width, new->height);
 }
 
 static void 
-panel_set_position (Panel *panel, XfceScreenPosition position,
-                    int xoffset, int yoffset)
+panel_set_position (Panel              *panel, 
+                    XfceScreenPosition  position,
+                    int                 xoffset, 
+                    int                 yoffset)
 {
-    PanelPrivate *priv;
-    GtkRequisition req;
-    int x, y;
-    XfceMonitor *xmon;
+    PanelPrivate   *priv;
+    XfceMonitor    *xmon;
+    GdkRectangle    rect;
+    GtkRequisition  req;
+    int             x, y;
 
     if (!GTK_WIDGET_VISIBLE (panel))
         return;
     
     priv = panel->priv;
-    
     xmon = panel_app_get_monitor (priv->monitor);
 
     priv->screen_position = position;
-
     _set_borders (panel);
     
     gtk_widget_size_request (GTK_WIDGET (panel), &req);
 
     x = y = 0;    
 
-    _calculate_coordinates (position, &(xmon->geometry), 
-                            req.width, req.height, &x, &y);
-    
-    if (G_UNLIKELY (xoffset < 0 || yoffset < 0))
+    /* NOTE: xoffset and yoffset are only used for floating panels */
+    if (xfce_screen_position_is_floating (position))
     {
-        gtk_window_get_position (GTK_WINDOW (panel), &xoffset, &yoffset);
-        
-        xoffset -= xmon->geometry.x;
-        yoffset -= xmon->geometry.y;
+        if (G_UNLIKELY (xoffset < 0 || yoffset < 0))
+        {
+            /* This means we switched to being a floating panel;
+             * calculate the actual position here. */
+            gtk_window_get_position (GTK_WINDOW (panel), &x, &y);
+        }
+        else
+        {
+            x = xmon->geometry.x + xoffset;
+            y = xmon->geometry.y + yoffset;
+        }
     }
     
-    switch (priv->full_width)
+    if (priv->full_width < XFCE_PANEL_SPAN_MONITORS)
     {
-        case XFCE_PANEL_NORMAL_WIDTH:
-            x += xoffset;
-            y += yoffset;
-            break;
-            
-        case XFCE_PANEL_SPAN_MONITORS:
-            if (xfce_screen_position_is_horizontal (position))
-                x = 0;
-            else
-                y = 0;
-            /* fall through */
-            
-        case XFCE_PANEL_FULL_WIDTH:
-            xoffset = yoffset = 0;
-            break;
-    }
+        _calculate_coordinates (position, 
+                                &(xmon->geometry), 
+                                req.width, 
+                                req.height, 
+                                &x, &y);
     
-    priv->xoffset = xoffset;
-    priv->yoffset = yoffset;
+        priv->xoffset = x - xmon->geometry.x;
+        priv->yoffset = y - xmon->geometry.y;
+    }
+    else
+    {
+        rect.x      = 0;
+        rect.y      = 0;
+        rect.width  = gdk_screen_get_width (xmon->screen);
+        rect.height = gdk_screen_get_height (xmon->screen);
+
+        _calculate_coordinates (position, 
+                                &rect,
+                                req.width, 
+                                req.height, 
+                                &x, &y);
+    
+        priv->xoffset = x;
+        priv->yoffset = y;
+    }
     
     DBG ("\n + Position: %d\n + Offset: (%d, %d)", 
          priv->screen_position, priv->xoffset, priv->yoffset);
@@ -464,17 +587,19 @@ panel_set_position (Panel *panel, XfceScreenPosition position,
 
 /* screen size */
 void 
-panel_screen_size_changed (GdkScreen *screen, Panel *panel)
+panel_screen_size_changed (GdkScreen *screen, 
+                           Panel     *panel)
 {
-    PanelPrivate *priv;
-    XfceMonitor *xmon;
-    XfcePanelWidthType width;
+    PanelPrivate       *priv;
+    XfceMonitor        *xmon;
+    XfcePanelWidthType  width;
 
     priv = panel->priv;
 
     xmon = panel_app_get_monitor (priv->monitor);
     
-    gdk_screen_get_monitor_geometry (screen, xmon->num, 
+    gdk_screen_get_monitor_geometry (screen, 
+                                     xmon->num, 
                                      &(xmon->geometry));
 
     /* new size constraints */
@@ -491,10 +616,11 @@ panel_screen_size_changed (GdkScreen *screen, Panel *panel)
 
 /* transparency and autohide */
 static void
-_set_transparent (Panel *panel, gboolean transparent)
+_set_transparent (Panel    *panel, 
+                  gboolean  transparent)
 {
     PanelPrivate *priv;
-    guint opacity;
+    guint         opacity;
 
     if (!GTK_WIDGET (panel)->window)
         return;
@@ -504,10 +630,14 @@ _set_transparent (Panel *panel, gboolean transparent)
     if (G_UNLIKELY (priv->opacity == 0))
     {
         if (priv->transparency != 0)
+        {
             priv->opacity =  
                 OPAQUE - rint ((double)priv->transparency * OPAQUE / 100);
+        }
         else
+        {
             priv->opacity = OPAQUE;
+        }
     }
     
     opacity = (transparent || priv->activetrans) ? priv->opacity : OPAQUE;
@@ -521,7 +651,9 @@ _set_transparent (Panel *panel, gboolean transparent)
         gdk_property_change (GTK_WIDGET (panel)->window,
                              gdk_atom_intern ("_NET_WM_WINDOW_OPACITY", FALSE),
                              gdk_atom_intern ("CARDINAL", FALSE), 32,
-                             GDK_PROP_MODE_REPLACE, (guchar *) & opacity, 1L);
+                             GDK_PROP_MODE_REPLACE, 
+                             (guchar *) & opacity, 
+                             1L);
 
         gdk_error_trap_pop ();
 
@@ -530,14 +662,16 @@ _set_transparent (Panel *panel, gboolean transparent)
 }
 
 void
-panel_set_hidden (Panel *panel, gboolean hide)
+panel_set_hidden (Panel    *panel, 
+                  gboolean  hide)
 {
     PanelPrivate *priv;
-    int w = 0, h = 0;
+    int           w, h;
 
     priv = panel->priv;
 
     priv->hidden = hide;
+    w = h = 0;
     
     if (hide)
     {
@@ -621,8 +755,76 @@ _unhide_timeout (Panel *panel)
     return FALSE;
 }
 
+static gboolean
+drag_motion (Panel          *panel, 
+             GdkDragContext *context,
+             int             x, 
+             int             y, 
+             guint           time, 
+             gpointer        user_data)
+{
+    PanelPrivate *priv;
+
+    priv = panel->priv;
+
+    if (!priv->hidden || priv->block_autohide)
+        return TRUE;
+
+    if (priv->hide_timeout)
+    {
+        g_source_remove (priv->hide_timeout);
+        priv->hide_timeout = 0;
+    }
+
+    if (!priv->unhide_timeout)
+    {
+        priv->unhide_timeout =
+            g_timeout_add (UNHIDE_TIMEOUT,
+                           (GSourceFunc) _unhide_timeout, panel);
+    }
+
+    return TRUE;
+}
+
+static void
+drag_leave (Panel          *panel,
+            GdkDragContext *drag_context,
+            guint           time,
+            gpointer        user_data)
+{
+    int    x, y, w, h, px, py;
+    PanelPrivate *priv;
+
+    priv = panel->priv;
+
+    if (priv->hidden || priv->block_autohide)
+        return;
+
+    /* check if pointer is inside the window */
+    gdk_display_get_pointer (gdk_display_get_default (), NULL, &px, &py, NULL);
+    gtk_window_get_position (GTK_WINDOW (panel), &x, &y);
+    gtk_window_get_size (GTK_WINDOW (panel), &w, &h);
+
+    if (px < x || px > x + w || py < y || py > y + h)
+    {
+        if (priv->unhide_timeout)
+        {
+            g_source_remove (priv->unhide_timeout);
+            priv->unhide_timeout = 0;
+        }
+
+        if (!priv->hide_timeout)
+        {
+            priv->hide_timeout =
+                g_timeout_add (HIDE_TIMEOUT,
+                               (GSourceFunc) _hide_timeout, panel);
+        }
+    }
+}
+
 static void 
-panel_enter (Panel *panel, GdkEventCrossing * event)
+panel_enter (Panel            *panel, 
+             GdkEventCrossing *event)
 {
     PanelPrivate *priv;
 
@@ -655,7 +857,8 @@ panel_enter (Panel *panel, GdkEventCrossing * event)
 
 
 static void 
-panel_leave (Panel *panel, GdkEventCrossing * event)
+panel_leave (Panel            *panel, 
+             GdkEventCrossing *event)
 {
     PanelPrivate *priv;
 
@@ -688,13 +891,15 @@ static void
 _window_mapped (Panel *panel)
 {
     PanelPrivate *priv;
-    XfceMonitor *xmon;
-    int x, y;
+    XfceMonitor  *xmon;
+    int           x, y;
 
     priv = panel->priv;
 
-    panel_set_position (panel, priv->screen_position, 
-                        priv->xoffset, priv->yoffset);
+    panel_set_position (panel, 
+                        priv->screen_position, 
+                        priv->xoffset, 
+                        priv->yoffset);
     
     _set_transparent (panel, TRUE);
 
@@ -710,9 +915,10 @@ _window_mapped (Panel *panel)
          priv->screen_position, priv->xoffset, priv->yoffset);
 
     if (priv->autohide)
+    {
         priv->hide_timeout =
             g_timeout_add (2000, (GSourceFunc)_hide_timeout, panel);
-    
+    }
 }
 
 /* public API */
@@ -729,16 +935,21 @@ void panel_init_signals (Panel *panel)
     g_signal_connect (panel, "map", G_CALLBACK (_window_mapped), NULL);
     
     g_signal_connect (panel, "move-end", G_CALLBACK (panel_move_end), NULL);
+
+    panel_dnd_set_dest (GTK_WIDGET (panel));
+    g_signal_connect (panel, "drag-motion", G_CALLBACK (drag_motion), NULL);
+    g_signal_connect (panel, "drag-leave", G_CALLBACK (drag_leave), NULL);
 }
 
 void 
 panel_init_position (Panel *panel)
 {
-    PanelPrivate *priv;
-    GtkOrientation orientation;
-    XfceMonitor *xmon;
-    int x, y;
-    GtkRequisition req;
+    PanelPrivate   *priv;
+    GtkOrientation  orientation;
+    XfceMonitor    *xmon;
+    int             x, y;
+    GtkRequisition  req;
+    GdkRectangle    rect;
 
     priv = panel->priv;
     
@@ -789,16 +1000,41 @@ panel_init_position (Panel *panel)
     x = xmon->geometry.x;
     y = xmon->geometry.y;
 
-    if (priv->xoffset > 0 && priv->yoffset > 0)
+    if (priv->xoffset > 0 || priv->yoffset > 0)
     {
         x += priv->xoffset;
         y += priv->yoffset;
     }
     else
     {
-        gtk_widget_size_request (GTK_WIDGET (panel), &req);
-        _calculate_coordinates (priv->screen_position, &(xmon->geometry), 
-                                req.width, req.height, &x, &y);
+        if (priv->full_width < XFCE_PANEL_SPAN_MONITORS)
+        {
+            gtk_widget_size_request (GTK_WIDGET (panel), &req);
+            _calculate_coordinates (priv->screen_position, 
+                                    &(xmon->geometry), 
+                                    req.width, 
+                                    req.height, 
+                                    &x, &y);
+    
+            priv->xoffset = x - xmon->geometry.x;
+            priv->yoffset = y - xmon->geometry.y;
+        }
+        else
+        {
+            rect.x      = 0;
+            rect.y      = 0;
+            rect.width  = gdk_screen_get_width (xmon->screen);
+            rect.height = gdk_screen_get_height (xmon->screen);
+
+            _calculate_coordinates (priv->screen_position, 
+                                    &rect,
+                                    req.width, 
+                                    req.height, 
+                                    &x, &y);
+    
+            priv->xoffset = x;
+            priv->yoffset = y;
+        }
     }
     
     DBG (" + offsets: (%d, %d)\n", 
@@ -818,9 +1054,9 @@ panel_init_position (Panel *panel)
 void 
 panel_center (Panel *panel)
 {
-    PanelPrivate *priv;
-    GtkRequisition req;
-    XfceMonitor *xmon;
+    PanelPrivate   *priv;
+    GtkRequisition  req;
+    XfceMonitor    *xmon;
 
     priv = panel->priv;
 
@@ -838,7 +1074,8 @@ panel_center (Panel *panel)
          priv->screen_position, priv->xoffset, priv->yoffset);
 }
 
-void panel_set_autohide (Panel *panel, gboolean autohide)
+void panel_set_autohide (Panel    *panel, 
+                         gboolean  autohide)
 {
     PanelPrivate *priv;
 
@@ -874,8 +1111,8 @@ void panel_set_autohide (Panel *panel, gboolean autohide)
     }        
     else
     {
-        int x, y, w, h;
         XfceMonitor *xmon;
+        int          x, y, w, h;
 
         gtk_window_get_position (GTK_WINDOW (panel), &x, &y);
         gtk_window_get_size (GTK_WINDOW (panel), &w, &h);
@@ -936,7 +1173,8 @@ void panel_unblock_autohide (Panel *panel)
 }
 
 void
-panel_set_full_width (Panel *panel, int fullwidth)
+panel_set_full_width (Panel *panel, 
+                      int    fullwidth)
 {
     PanelPrivate *priv;
 
@@ -949,9 +1187,10 @@ panel_set_full_width (Panel *panel, int fullwidth)
         if (GTK_WIDGET_VISIBLE (panel))
         {
             XfceMonitor *xmon;
-            int w = 0, h = 0;
+            int          w, h;
 
             xmon = panel_app_get_monitor (priv->monitor);
+            w = h = 0;
 
             switch (priv->full_width)
             {
@@ -992,7 +1231,8 @@ panel_set_full_width (Panel *panel, int fullwidth)
 }
 
 void
-panel_set_transparency (Panel *panel, int transparency)
+panel_set_transparency (Panel *panel, 
+                        int    transparency)
 {
     PanelPrivate *priv;
 
@@ -1003,14 +1243,15 @@ panel_set_transparency (Panel *panel, int transparency)
         DBG ("Transparency: %d", transparency);
 
         priv->transparency = transparency;
-        priv->opacity = 0;
+        priv->opacity      = 0;
 
         _set_transparent (panel, TRUE);
     }
 }
 
 void
-panel_set_activetrans (Panel *panel, gboolean activetrans)
+panel_set_activetrans (Panel    *panel, 
+                       gboolean  activetrans)
 {
     PanelPrivate *priv = panel->priv;
 
@@ -1037,7 +1278,8 @@ panel_get_size (Panel *panel)
 }
 
 void 
-panel_set_size (Panel *panel, int size)
+panel_set_size (Panel *panel, 
+                int    size)
 {
     PanelPrivate *priv;
 
@@ -1068,7 +1310,8 @@ panel_get_monitor (Panel *panel)
 }
 
 void 
-panel_set_monitor (Panel *panel, int monitor)
+panel_set_monitor (Panel *panel, 
+                   int    monitor)
 {
     PanelPrivate *priv;
 
@@ -1078,8 +1321,8 @@ panel_set_monitor (Panel *panel, int monitor)
 
     if (monitor != priv->monitor)
     {
-        XfceMonitor *xmon;
-        XfcePanelWidthType width;
+        XfceMonitor        *xmon;
+        XfcePanelWidthType  width;
         
         /* TODO: check range */
         priv->monitor = monitor;
@@ -1119,7 +1362,8 @@ panel_get_screen_position (Panel *panel)
 }
 
 void 
-panel_set_screen_position (Panel *panel, XfceScreenPosition position)
+panel_set_screen_position (Panel              *panel, 
+                           XfceScreenPosition  position)
 {
     PanelPrivate *priv;
 
@@ -1129,9 +1373,12 @@ panel_set_screen_position (Panel *panel, XfceScreenPosition position)
 
     if (position != priv->screen_position)
     {
-        GtkOrientation orientation;
+        GtkOrientation     orientation;
         XfcePanelWidthType full_width = priv->full_width;
         
+        /* use edit_mode property to test if we are changing position */
+        priv->edit_mode = TRUE;
+
         xfce_panel_window_set_move_function (XFCE_PANEL_WINDOW (panel),
                                              NULL, NULL);
 
@@ -1195,7 +1442,9 @@ panel_set_screen_position (Panel *panel, XfceScreenPosition position)
             panel_set_full_width (panel, full_width);
         }
 
-        gtk_widget_queue_draw (GTK_WIDGET (panel));
+        gtk_widget_queue_resize (GTK_WIDGET (panel));
+
+        g_idle_add((GSourceFunc)unblock_struts, panel);
     }
 }
 
@@ -1212,7 +1461,8 @@ panel_get_xoffset (Panel *panel)
 }
 
 void 
-panel_set_xoffset (Panel *panel, int xoffset)
+panel_set_xoffset (Panel *panel, 
+                   int    xoffset)
 {
     PanelPrivate *priv;
 
@@ -1241,7 +1491,8 @@ panel_get_yoffset (Panel *panel)
 }
 
 void 
-panel_set_yoffset (Panel *panel, int yoffset)
+panel_set_yoffset (Panel *panel, 
+                   int    yoffset)
 {
     PanelPrivate *priv;
 
