@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: clock.c 22739 2006-08-13 09:10:57Z jasper $
+/*  $Id: clock.c 24548 2007-01-19 16:56:52Z nick $
  *
  *  Copyright © 2005 The Xfce Development Team
  *
@@ -61,6 +61,7 @@ typedef struct
     GtkTooltips *tips;
 
     int timeout_id;
+    int screen_changed_id;
 
     /* Settings */
     int mode;
@@ -155,7 +156,7 @@ clock_update_size (Clock *clock, int size)
     /* keep in sync with systray */
     if (size > 26)
     {
-        gtk_container_set_border_width (GTK_CONTAINER (clock->frame), 2);
+        gtk_container_set_border_width (GTK_CONTAINER (clock->frame), 1);
         size -= 3;
     }
     else
@@ -169,22 +170,31 @@ clock_update_size (Clock *clock, int size)
      * can use them here (e.g. 10*2 => DIGIT_SMALL_HEIGHT*2)
      * */
 
-    if (size <= 10*2)
+    if (xfce_panel_plugin_get_orientation (clock->plugin) 
+            == GTK_ORIENTATION_HORIZONTAL)
     {
+        if (size <= 10*2)
+        {
+            xfce_clock_set_led_size (clk, DIGIT_SMALL);
+        }
+        else if (size <= 14*2) 
+        {
+            xfce_clock_set_led_size (clk, DIGIT_MEDIUM);
+        } 
+        else if (size <= 20*2) 
+        {
+            xfce_clock_set_led_size (clk, DIGIT_LARGE);
+        }
+        else 
+        {
+            xfce_clock_set_led_size (clk, DIGIT_HUGE);
+        }
+    }
+    else
+    {
+        /* Always use small size in vertical mode */
         xfce_clock_set_led_size (clk, DIGIT_SMALL);
     }
-    else if (size <= 14*2) 
-    {
-        xfce_clock_set_led_size (clk, DIGIT_MEDIUM);
-    } 
-    else if (size <= 20*2) 
-    {
-        xfce_clock_set_led_size (clk, DIGIT_LARGE);
-    }
-    else 
-    {
-        xfce_clock_set_led_size (clk, DIGIT_HUGE);
-    }
 
     if ((xfce_clock_get_mode (clk) == XFCE_CLOCK_LEDS) ||
         (xfce_clock_get_mode (clk) == XFCE_CLOCK_DIGITAL))
@@ -226,6 +236,8 @@ clock_free_data (XfcePanelPlugin *plugin, Clock *clock)
 
     if (dlg)
         gtk_widget_destroy (dlg);
+        
+    g_signal_handler_disconnect (plugin, clock->screen_changed_id);
     
     g_source_remove (clock->timeout_id);
     g_object_unref (G_OBJECT (clock->tips));
@@ -280,6 +292,25 @@ clock_read_rc_file (XfcePanelPlugin *plugin, Clock* clock)
             xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin)));
 }
 
+static void
+clock_screen_changed (GtkWidget *plugin, GdkScreen *screen, 
+                      Clock *clock)
+{
+	/* return when the widget is not visible (when moving the plugin) */
+	if (GTK_IS_INVISIBLE (clock->clock) == FALSE)
+	    return;
+	
+	gtk_widget_destroy (clock->clock);
+	
+	clock->clock = xfce_clock_new ();
+    gtk_widget_show (clock->clock);
+    gtk_container_add (GTK_CONTAINER (clock->frame), clock->clock);
+
+    xfce_clock_set_interval (XFCE_CLOCK (clock->clock), 1000);
+    
+    clock_read_rc_file (clock->plugin, clock);
+}
+
 static void
 clock_write_rc_file (XfcePanelPlugin *plugin, Clock *clock)
 {
@@ -325,6 +356,10 @@ clock_construct (XfcePanelPlugin *plugin)
     xfce_panel_plugin_menu_show_configure (plugin);
     g_signal_connect (plugin, "configure-plugin", 
                       G_CALLBACK (clock_properties_dialog), clock);
+                      
+    clock->screen_changed_id = 
+        g_signal_connect (plugin, "screen-changed", 
+                          G_CALLBACK (clock_screen_changed), clock);
 
     clock->frame = gtk_frame_new (NULL);
     gtk_widget_show (clock->frame);
@@ -448,6 +483,9 @@ clock_properties_dialog (XfcePanelPlugin *plugin, Clock *clock)
                 GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
                 NULL);
 
+    gtk_window_set_screen (GTK_WINDOW (dlg), 
+                           gtk_widget_get_screen (GTK_WIDGET (plugin)));
+
     g_object_set_data (G_OBJECT (plugin), "dialog", dlg);
 
     gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
