@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: launcher.h 21876 2006-05-28 15:41:24Z jasper $
+/*  $Id: launcher.h 23356 2006-10-11 19:10:40Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -24,15 +24,12 @@
 
 #define MENU_TIMEOUT               250
 #define BORDER                     8
-#define W_ARROW                    16
+#define W_ARROW                    12
 #define MENU_ICON_SIZE             24
 #define DLG_ICON_SIZE              32
 #define PANEL_ICON_SIZE            48
 #define MIN_ICON_SIZE              12 
 
-/* A bit of a hack: Don't use icons without xfce- default */
-#define NUM_CATEGORIES             (XFCE_N_BUILTIN_ICON_CATEGORIES - 3)
-
 typedef enum    _LauncherIconType  LauncherIconType;
 typedef struct  _LauncherIcon      LauncherIcon;
 typedef struct  _LauncherEntry     LauncherEntry;
