@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: pager.c 21756 2006-05-21 12:59:15Z jasper $
+/*  $Id: pager.c 24221 2006-12-30 14:08:01Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -34,14 +34,17 @@ typedef struct
     XfcePanelPlugin *plugin;
 
     NetkScreen *screen;
+    GdkScreen *gdk_screen;
+    GtkWidget *pager;
+
     
     int ws_created_id;
     int ws_destroyed_id;
     int screen_changed_id;
+    int screen_size_changed_id;
 
     int rows;
-    
-    GtkWidget *pager;
+    guint scrolling:1;
 }
 Pager;
 
@@ -109,8 +112,13 @@ pager_free_data (XfcePanelPlugin *plugin, Pager *pager)
         pager->ws_destroyed_id = 0;
     }
 
-    
-    g_free (pager);
+    if (pager->screen_size_changed_id)
+    {
+        g_signal_handler_disconnect (pager->gdk_screen, pager->screen_size_changed_id);
+        pager->screen_size_changed_id = 0;
+    }
+
+    panel_slice_free (Pager, pager);
 }
 
 static void
@@ -119,6 +127,7 @@ pager_read_rc_file (XfcePanelPlugin *plugin, Pager *pager)
     char *file;
     XfceRc *rc;
     int rows = 1;
+    gboolean scrolling = TRUE;
     
     if ((file = xfce_panel_plugin_lookup_rc_file (plugin)) != NULL)
     {
@@ -128,10 +137,12 @@ pager_read_rc_file (XfcePanelPlugin *plugin, Pager *pager)
         if (rc != NULL)
         {
             rows = xfce_rc_read_int_entry (rc, "rows", 1);
+	    scrolling = xfce_rc_read_bool_entry (rc, "scrolling", TRUE);
         }
     }
 
     pager->rows = rows;
+    pager->scrolling = scrolling;
 }
 
 static void
@@ -150,6 +161,8 @@ pager_write_rc_file (XfcePanelPlugin *plugin, Pager *pager)
         return;
     
     xfce_rc_write_int_entry (rc, "rows", pager->rows);
+    
+    xfce_rc_write_bool_entry (rc, "scrolling", pager->scrolling);
 
     xfce_rc_close (rc);
 }
@@ -163,9 +176,13 @@ pager_n_workspaces_changed (NetkScreen * screen, NetkWorkspace * ws,
                     xfce_panel_plugin_get_size (pager->plugin));
 }
 
+static void pager_screen_size_changed (GdkScreen *screen, Pager *pager);
+
 static void
 pager_screen_changed (GtkWidget *plugin, GdkScreen *screen, Pager *pager)
 {
+    screen = gtk_widget_get_screen (plugin);
+
     if (!screen)
         return;
 
@@ -181,6 +198,13 @@ pager_screen_changed (GtkWidget *plugin, GdkScreen *screen, Pager *pager)
         pager->ws_destroyed_id = 0;
     }
 
+    if (pager->screen_size_changed_id)
+    {
+        g_signal_handler_disconnect (pager->gdk_screen, pager->screen_size_changed_id);
+        pager->screen_size_changed_id = 0;
+    }
+
+    pager->gdk_screen = screen;
     pager->screen = netk_screen_get (gdk_screen_get_number (screen));
 
     netk_pager_set_screen (NETK_PAGER (pager->pager), pager->screen);
@@ -192,14 +216,25 @@ pager_screen_changed (GtkWidget *plugin, GdkScreen *screen, Pager *pager)
     pager->ws_destroyed_id =
 	g_signal_connect (pager->screen, "workspace-destroyed",
 			  G_CALLBACK (pager_n_workspaces_changed), pager);
+
+    pager->screen_size_changed_id = 
+        g_signal_connect (screen, "size-changed", 
+                          G_CALLBACK (pager_screen_size_changed), pager);
 }
-        
+
+static void
+pager_screen_size_changed (GdkScreen *screen, Pager *pager)
+{
+    pager_screen_changed (GTK_WIDGET (pager->plugin), screen, pager);
+    gtk_widget_queue_resize (GTK_WIDGET (pager->plugin));
+}
+
 static void 
 pager_construct (XfcePanelPlugin *plugin)
 {
     GdkScreen *screen;
     int screen_idx;
-    Pager *pager = g_new0 (Pager, 1);
+    Pager *pager = panel_slice_new0 (Pager);
 
     g_signal_connect (plugin, "orientation-changed", 
                       G_CALLBACK (pager_orientation_changed), pager);
@@ -219,7 +254,7 @@ pager_construct (XfcePanelPlugin *plugin)
 
     pager->plugin = plugin;
 
-    screen = gtk_widget_get_screen (GTK_WIDGET (plugin));
+    pager->gdk_screen = screen = gtk_widget_get_screen (GTK_WIDGET (plugin));
     screen_idx = gdk_screen_get_number (screen);
     pager->screen = netk_screen_get (screen_idx);
     
@@ -229,6 +264,7 @@ pager_construct (XfcePanelPlugin *plugin)
     netk_pager_set_orientation (NETK_PAGER (pager->pager), 
                                 xfce_panel_plugin_get_orientation (plugin));
     netk_pager_set_n_rows (NETK_PAGER (pager->pager), pager->rows);
+    netk_pager_set_workspace_scrolling (NETK_PAGER (pager->pager), pager->scrolling);
     gtk_widget_show (pager->pager);
     gtk_container_add (GTK_CONTAINER (plugin), pager->pager);
     
@@ -245,6 +281,10 @@ pager_construct (XfcePanelPlugin *plugin)
     pager->screen_changed_id = 
         g_signal_connect (plugin, "screen-changed", 
                           G_CALLBACK (pager_screen_changed), pager);
+
+    pager->screen_size_changed_id = 
+        g_signal_connect (screen, "size-changed", 
+                          G_CALLBACK (pager_screen_size_changed), pager);
 }
 
 /* -------------------------------------------------------------------- *
@@ -264,6 +304,19 @@ rows_changed (GtkSpinButton * spin, Pager * pager)
     }
 }
 
+static void
+workspace_scrolling_toggled (GtkWidget *button, Pager *pager)
+{
+    gboolean scrolling = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+    
+    if (pager->scrolling != scrolling)
+    {
+	pager->scrolling = scrolling;
+	
+	netk_pager_set_workspace_scrolling (NETK_PAGER (pager->pager), scrolling);
+    }
+}
+
 static void
 pager_dialog_response (GtkWidget *dlg, int reponse, Pager *pager)
 {
@@ -277,13 +330,12 @@ pager_dialog_response (GtkWidget *dlg, int reponse, Pager *pager)
 static void
 pager_properties_dialog (XfcePanelPlugin *plugin, Pager *pager)
 {
-    GtkWidget *dlg, *vbox, *hbox, *label, *spin;
+    GtkWidget *dlg, *vbox, *hbox, *label, *spin, *scrolling;
     int max;
 
     xfce_panel_plugin_block_menu (plugin);
     
-    dlg = xfce_titled_dialog_new_with_buttons (_("Pager"),
-                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
+    dlg = xfce_titled_dialog_new_with_buttons (_("Pager"), NULL,
                 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
                 GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
                 NULL);
@@ -293,6 +345,9 @@ pager_properties_dialog (XfcePanelPlugin *plugin, Pager *pager)
     gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
     gtk_window_set_icon_name (GTK_WINDOW (dlg), "xfce4-settings");
     
+    gtk_window_set_screen (GTK_WINDOW (dlg), 
+                           gtk_widget_get_screen (GTK_WIDGET (plugin)));
+
     g_signal_connect (dlg, "response", G_CALLBACK (pager_dialog_response),
                       pager);
 
@@ -340,6 +395,14 @@ pager_properties_dialog (XfcePanelPlugin *plugin, Pager *pager)
         gtk_widget_show (label);
         gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
     }
+    
+    scrolling = gtk_check_button_new_with_mnemonic (_("Switch workspaces using the mouse wheel"));
+    gtk_widget_show (scrolling);
+    gtk_box_pack_start (GTK_BOX (vbox), scrolling, FALSE, FALSE, 0);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scrolling), pager->scrolling);
+    
+    g_signal_connect (scrolling, "toggled",
+        G_CALLBACK (workspace_scrolling_toggled), pager);
   
     gtk_widget_show (dlg);
 }
