@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: systray.c 21756 2006-05-21 12:59:15Z jasper $
+/*  $Id: systray.c 24221 2006-12-30 14:08:01Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -195,11 +195,11 @@ systray_orientation_changed (XfcePanelPlugin *plugin,
 static gboolean 
 systray_set_size (XfcePanelPlugin *plugin, int size, Systray *systray)
 {
-    int border = size > 26 ? 2 : 0;
+    int border = size > 26 ? 1 : 0;
 
     gtk_container_set_border_width (GTK_CONTAINER (systray->frame), border);
     
-    size = size - border - 2 - MAX (systray->frame->style->xthickness,
+    size = size - (2*border) - 2 - MAX (systray->frame->style->xthickness,
                                     systray->frame->style->ythickness);
     
     if (xfce_panel_plugin_get_orientation (plugin) 
@@ -224,7 +224,7 @@ systray_free_data (XfcePanelPlugin *plugin, Systray *systray)
         gtk_widget_destroy (dlg);
     
     systray_stop (systray);
-    g_free (systray);
+    panel_slice_free (Systray, systray);
 }
 
 static void
@@ -274,7 +274,7 @@ systray_write_rc_file (XfcePanelPlugin *plugin, Systray *systray)
 static void 
 systray_construct (XfcePanelPlugin *plugin)
 {
-    Systray *systray = g_new0 (Systray, 1);
+    Systray *systray = panel_slice_new0 (Systray);
 
     g_signal_connect (plugin, "orientation-changed", 
                       G_CALLBACK (systray_orientation_changed), systray);
@@ -375,12 +375,14 @@ systray_properties_dialog (XfcePanelPlugin *plugin, Systray *systray)
 
     xfce_panel_plugin_block_menu (plugin);
     
-    dlg = xfce_titled_dialog_new_with_buttons (_("System Tray"),
-                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
+    dlg = xfce_titled_dialog_new_with_buttons (_("System Tray"), NULL,
                 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
                 GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
                 NULL);
 
+    gtk_window_set_screen (GTK_WINDOW (dlg), 
+                           gtk_widget_get_screen (GTK_WIDGET (plugin)));
+
     g_object_set_data (G_OBJECT (plugin), "dialog", dlg);
 
     gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
