@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id$
+/*  $Id: windowlist.c 24548 2007-01-19 16:56:52Z nick $
  *
  *  Copyright (c) 2003 Andre Lerche <a.lerche@gmx.net>
  *  Copyright (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
@@ -37,11 +37,18 @@
 #include <config.h>
 #endif
 
+#include <string.h>
+#include <stdlib.h>
+#include <gdk/gdkkeysyms.h>
+
 #include <libxfce4panel/xfce-arrow-button.h>
 #include <libxfcegui4/netk-window-action-menu.h>
 
 #include "windowlist.h"
 #include "windowlist-dialog.h"
+#include "xfce4-popup-windowlist.h"
+
+typedef enum { WS_ADD = 1, WS_REMOVE } WorkspaceAction;
 
 static gboolean
 windowlist_blink (gpointer data);
@@ -64,8 +71,8 @@ XFCE_PANEL_PLUGIN_REGISTER_INTERNAL (windowlist_construct);
 /**
  * Common functions
  **/
-static gchar *
-menulist_utf8_string (gchar *string)
+static char *
+menulist_utf8_string (const char *string)
 {
     char *utf8 = NULL;
     if (!g_utf8_validate(string, -1, NULL))
@@ -77,7 +84,10 @@ menulist_utf8_string (gchar *string)
     {
         utf8 = g_strdup (string);
     }
-    
+
+    if (!utf8)
+        utf8 = g_strdup ("?");
+
     return utf8;
 }
 
@@ -93,7 +103,7 @@ menulist_workspace_name (NetkWorkspace *workspace,
     ws_num  = netk_workspace_get_number (workspace);    
     ws_name = netk_workspace_get_name (workspace);
 	    
-    if(!ws_name || atoi(ws_name) == ws_num + 1)
+    if(!ws_name || atoi((const char *)ws_name) == ws_num + 1)
 	ws_title = g_strdup_printf(num_title, ws_num + 1);
     else
 	ws_title = g_markup_printf_escaped(name_title, ws_name);
@@ -149,10 +159,10 @@ mi_destroyed (GtkObject *object,
 
 static void
 action_menu_deactivated (GtkMenu *menu,
-			 GtkMenu *parent)
+                         GtkMenu *parent)
 {
     DBG ("Destroy Action Menu");
-    
+
     gtk_menu_popdown (parent);
     g_signal_emit_by_name (parent, "deactivate", 0);
 }
@@ -168,11 +178,11 @@ popup_action_menu (GtkWidget * widget,
 
     menu = netk_create_window_action_menu (window);
 
+    g_signal_connect (menu, "deactivate",
+                      G_CALLBACK (action_menu_deactivated), widget->parent);
+
     gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0,
                     GDK_CURRENT_TIME);
-
-    g_signal_connect (menu, "deactivate", 
-                      G_CALLBACK (action_menu_deactivated), widget->parent);
 }
 
 static gboolean
@@ -182,17 +192,23 @@ menulist_goto_window (GtkWidget *mi,
 {
     if (ev->button == 1) /* Goto workspace and show window */
     {
+        gtk_menu_popdown (GTK_MENU (mi->parent));
         netk_workspace_activate(netk_window_get_workspace(window));
 	netk_window_activate (window);
+        g_signal_emit_by_name (mi->parent, "deactivate", 0);
     }
     else if (ev->button == 2) /* Show window on current workspace */
     {
+        gtk_menu_popdown (GTK_MENU (mi->parent));
 	netk_window_activate (window);
+        g_signal_emit_by_name (mi->parent, "deactivate", 0);
     }
     else if (ev->button == 3) /* Show the popup menu */
     {
         popup_action_menu (mi, window);
-        return TRUE;
+
+	/* the right-click menu will pop down the main menu */
+	return TRUE;
     }
 
     return FALSE;
@@ -237,6 +253,87 @@ menulist_remove_screen (GtkWidget *mi,
     return FALSE;
 }
 
+static gboolean
+menulist_keypress (GtkWidget *menu, GdkEventKey *ev, Windowlist *wl)
+{
+    GdkEventButton evb;
+    GList *l;
+    GtkWidget *mi = NULL;
+    NetkWindow *window;
+    NetkWorkspace *workspace;
+    gpointer ws_action;
+    guint state;
+
+    for (l = GTK_MENU_SHELL (menu)->children; l != NULL; l = l->next)
+    {
+        if (GTK_WIDGET_STATE (l->data) == GTK_STATE_PRELIGHT)
+        {
+            mi = l->data;
+            break;
+        }
+    }
+
+    if (mi == NULL)
+        return FALSE;
+
+    state = ev->state & gtk_accelerator_get_default_mod_mask ();
+
+    switch (ev->keyval)
+    {
+        case GDK_space:
+        case GDK_Return:
+        case GDK_KP_Space:
+        case GDK_KP_Enter:
+            evb.button = 1;
+            break;
+        case GDK_Menu:
+            evb.button = 3;
+            break;
+        default:
+            return FALSE;
+    }
+
+    if (evb.button == 1)
+    {
+        if (state == GDK_SHIFT_MASK)
+            evb.button = 2;
+        else if (state == GDK_CONTROL_MASK)
+            evb.button = 3;
+    }
+
+    if((window = g_object_get_data (G_OBJECT (mi), "netk-window")) != NULL)
+    {
+        if (!NETK_IS_WINDOW (window))
+            return FALSE;
+
+        return menulist_goto_window (mi, &evb, window);
+    }
+    else if (evb.button == 1 && 
+             (workspace = g_object_get_data (G_OBJECT (mi), "netk-workspace")) 
+                != NULL)
+    {
+        if (!NETK_IS_WORKSPACE (workspace))
+            return FALSE;
+
+        return menulist_goto_workspace (mi, NULL, workspace);
+    }
+    else if (evb.button == 1 && 
+             (ws_action = g_object_get_data (G_OBJECT (mi), "ws-action"))
+                != NULL)
+    {
+        if (GPOINTER_TO_INT (ws_action) == WS_REMOVE)
+        {
+            return menulist_remove_screen (mi, NULL, wl);
+        }
+        else 
+        {
+            return menulist_add_screen (mi, NULL, wl);
+        }
+    }
+
+    return FALSE;
+}
+
 /**
  * Menu position function
  **/
@@ -277,6 +374,8 @@ windowlist_position_menu (GtkMenu *menu,
             *x += widget->allocation.x + widget->allocation.width;
             *y += - req.height + widget->allocation.height;
             break;
+        default:
+            break;
     }
 
     screen = gtk_widget_get_screen (widget);
@@ -305,28 +404,20 @@ menulist_menu_item (NetkWindow *window,
 		    gint size)
 {
     GtkWidget *mi;
-    const char *window_name = NULL;
+    char *window_name = NULL;
     GString *label;
     GdkPixbuf *icon = NULL, *tmp = NULL;
-    gboolean truncated = FALSE;
     
-    window_name = netk_window_get_name (window);
+    window_name = menulist_utf8_string (netk_window_get_name (window));
     label = g_string_new (window_name);
     
-    if (label->len >= 20)
-    {
-        g_string_truncate (label, 20);
-        g_string_append (label, " ...");
-	truncated = TRUE;
-    }
-
     if (netk_window_is_minimized (window))
     {
         g_string_prepend (label, "[");
         g_string_append (label, "]");
     }
     
-    /* Italic fonts are not completely shown */
+    /* hack: italic fonts are not completely shown, otherwise */
     g_string_append (label, " ");
     
     if (wl->show_window_icons)
@@ -336,7 +427,6 @@ menulist_menu_item (NetkWindow *window,
     {
         GtkWidget *img;
         gint w, h;
-        char *text;
     
         w = gdk_pixbuf_get_width (icon);
         h = gdk_pixbuf_get_height (icon);
@@ -348,58 +438,47 @@ menulist_menu_item (NetkWindow *window,
             icon = tmp;
         }
     
-        text = menulist_utf8_string (label->str);
-        mi = gtk_image_menu_item_new_with_label (text ? text : "?");
-        g_free (text);
+        mi = gtk_image_menu_item_new_with_label (label->str);
         
         img = gtk_image_new_from_pixbuf (icon);
         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), img);
 	
 	if (G_LIKELY (tmp))
-	    g_object_unref (tmp);
+	    g_object_unref (G_OBJECT (tmp));
     }
     else
     {
-        char *text;
-    
-        text = menulist_utf8_string (label->str);
-        mi = gtk_menu_item_new_with_label (text ? text : "?");
-        g_free (text);
+        mi = gtk_menu_item_new_with_label (label->str);
     }
     
-    if (truncated)
-    {
-        char *text = menulist_utf8_string ((char *)window_name);
-    	gtk_tooltips_set_tip (wl->tooltips, mi, text ? text : "?", NULL);
-        g_free (text);
-    }
+    gtk_label_set_ellipsize (GTK_LABEL (GTK_BIN (mi)->child), 
+                             PANGO_ELLIPSIZE_END);
+    gtk_label_set_max_width_chars (GTK_LABEL (GTK_BIN (mi)->child), 24);
+    
+    gtk_tooltips_set_tip (wl->tooltips, mi, window_name, NULL);
 
     g_string_free (label, TRUE);
+    g_free (window_name);
     
     return mi;
 }
 
 static gboolean
-menulist_toggle_menu (GtkToggleButton *button,
-                      GdkEventButton *ev, 
-                      Windowlist * wl)
+menulist_popup_menu (Windowlist * wl,
+                     GdkEventButton *ev,
+                     gboolean at_pointer)
 {
     GtkWidget *menu, *mi, *icon;
     NetkWindow *window;
-    NetkWindowState state;
     NetkWorkspace *netk_workspace, *active_workspace, *window_workspace;
     gchar *ws_label, *rm_label;
     gint size, i, wscount;
     GList *windows, *li;
-    PangoFontDescription *italic, *bold, *bold_italic;
-    
-    if (ev->button != 1)
-	return FALSE;
+    PangoFontDescription *italic, *bold;
     
     /* Menu item styles */
     italic = pango_font_description_from_string ("italic");
     bold = pango_font_description_from_string ("bold");
-    bold_italic = pango_font_description_from_string ("bold italic");
     
     menu = gtk_menu_new ();
     xfce_panel_plugin_register_menu (XFCE_PANEL_PLUGIN (wl->plugin), 
@@ -431,6 +510,9 @@ menulist_toggle_menu (GtkToggleButton *button,
         mi = gtk_menu_item_new_with_label (ws_label);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 	g_free (ws_label);
+
+        g_object_set_data (G_OBJECT (mi), "netk-workspace", netk_workspace);
+
         g_signal_connect (mi, "button-release-event",
 		G_CALLBACK (menulist_goto_workspace), netk_workspace);
 	
@@ -459,9 +541,6 @@ menulist_toggle_menu (GtkToggleButton *button,
 		netk_window_is_skip_tasklist (window))
 		    continue;
 	    
-	    /* Get Workspace state */
-	    state = netk_window_get_state (window);
-	    
 	    /* Create menu item */
 	    mi = menulist_menu_item (window,
 				     wl,
@@ -470,33 +549,41 @@ menulist_toggle_menu (GtkToggleButton *button,
 	    if (G_UNLIKELY (!mi))
                 continue;
 	    
+	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+	    
+            if (netk_window_is_active (window))
+            {
+                gtk_widget_modify_font (gtk_bin_get_child (GTK_BIN (mi)), 
+			italic);
+            }
+
 	    /* Apply some styles for windows on !current workspace and 
              * if they are urgent */
-	    if(state & NETK_WINDOW_STATE_URGENT && 
-               netk_workspace == active_workspace)
-	    {
-		gtk_widget_modify_font (gtk_bin_get_child (GTK_BIN (mi)), 
-			bold);
-	    }
-	    else if (state & NETK_WINDOW_STATE_URGENT)
-	    {
-		gtk_widget_modify_fg (gtk_bin_get_child (GTK_BIN (mi)),
-                        GTK_STATE_NORMAL,
-                        &(menu->style->fg[GTK_STATE_INSENSITIVE]));
-		gtk_widget_modify_font (gtk_bin_get_child (GTK_BIN (mi)), 
-			bold_italic);
-	    }
+	    if (netk_window_or_transient_demands_attention (window)) 
+            {
+                if (netk_workspace == active_workspace)
+                {
+                    gtk_widget_modify_font (gtk_bin_get_child (GTK_BIN (mi)), 
+                                            bold);
+                }
+                else
+                {
+                    gtk_widget_modify_fg (gtk_bin_get_child (GTK_BIN (mi)),
+                             GTK_STATE_NORMAL,
+                             &(menu->style->fg[GTK_STATE_INSENSITIVE]));
+                    gtk_widget_modify_font (gtk_bin_get_child (GTK_BIN (mi)), 
+                                            bold);
+                }
+            }
 	    else if (netk_workspace != active_workspace)
             {
                 gtk_widget_modify_fg (gtk_bin_get_child (GTK_BIN (mi)),
                         GTK_STATE_NORMAL,
                         &(menu->style->fg[GTK_STATE_INSENSITIVE]));
-		gtk_widget_modify_font (gtk_bin_get_child (GTK_BIN (mi)), 
-			italic);
             }
-	    
-	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 	 
+            g_object_set_data (G_OBJECT (mi), "netk-window", window);
+
 	    /* Connect some signals */
 	    g_signal_connect (mi, "button-release-event",
                     G_CALLBACK (menulist_goto_window), window);
@@ -517,11 +604,12 @@ menulist_toggle_menu (GtkToggleButton *button,
     
     pango_font_description_free(italic);
     pango_font_description_free(bold);
-    pango_font_description_free(bold_italic);
-    
+
     /* Show add/remove workspace buttons */
     if (wl->show_workspace_actions)
     {
+        int ws_action;
+
 	mi = gtk_separator_menu_item_new ();
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 	
@@ -537,6 +625,9 @@ menulist_toggle_menu (GtkToggleButton *button,
 	    mi = gtk_menu_item_new_with_label (_("Add workspace"));
 	}
 	
+        ws_action = WS_ADD;
+        g_object_set_data (G_OBJECT (mi), "ws-action", GINT_TO_POINTER (ws_action));
+
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
 	g_signal_connect (mi, "button-release-event",
@@ -544,34 +635,47 @@ menulist_toggle_menu (GtkToggleButton *button,
 	
 	/* Remove workspace */
 	wscount = netk_screen_get_workspace_count (wl->screen);
-	netk_workspace = netk_screen_get_workspace (wl->screen, wscount-1);
-	
-	rm_label = menulist_workspace_name (netk_workspace,
-					    _("Remove Workspace %d"),
-					    _("Remove Workspace '%s'"));
-	
-	if (wl->show_window_icons)
-	{
-	    mi = gtk_image_menu_item_new_with_label (rm_label);
-	    icon = gtk_image_new_from_stock (GTK_STOCK_REMOVE, 
-                                             GTK_ICON_SIZE_MENU);
-	    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), icon);
-	}
-	else
-	{
-	    mi = gtk_menu_item_new_with_label (rm_label);
-	}
-	
-	g_free (rm_label);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
-	g_signal_connect (mi, "button-release-event",
-		G_CALLBACK (menulist_remove_screen), wl);
+        if (wscount > 1)
+        {
+            netk_workspace = netk_screen_get_workspace (wl->screen, wscount-1);
+            
+            rm_label = menulist_workspace_name (netk_workspace,
+                                                _("Remove Workspace %d"),
+                                                _("Remove Workspace '%s'"));
+            
+            if (wl->show_window_icons)
+            {
+                mi = gtk_image_menu_item_new_with_label (rm_label);
+                icon = gtk_image_new_from_stock (GTK_STOCK_REMOVE, 
+                                                 GTK_ICON_SIZE_MENU);
+                gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), icon);
+            }
+            else
+            {
+                mi = gtk_menu_item_new_with_label (rm_label);
+            }
 
+            g_free (rm_label);
+
+            ws_action = WS_REMOVE;
+            g_object_set_data (G_OBJECT (mi), "ws-action", GINT_TO_POINTER (ws_action));
+
+            gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+
+            g_signal_connect (mi, "button-release-event",
+                    G_CALLBACK (menulist_remove_screen), wl);
+        }
     }
     
+    /* key presses work on the menu, not the items */
+    g_signal_connect (menu, "key-press-event",
+            G_CALLBACK (menulist_keypress), 
+            wl);
+
     /* Activate toggle button */
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wl->button), TRUE);
+    if (!at_pointer)
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wl->button), TRUE);
     
     /* Connect signal, show widgets and popup */
     g_signal_connect (menu, "deactivate",
@@ -580,12 +684,23 @@ menulist_toggle_menu (GtkToggleButton *button,
     gtk_widget_show_all (menu);
     
     gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 
-                    (GtkMenuPositionFunc)windowlist_position_menu,
-		    wl, 0, ev->time);
+                    at_pointer ? NULL
+                               : (GtkMenuPositionFunc)windowlist_position_menu,
+		    wl, 0, ev ? ev->time : GDK_CURRENT_TIME);
     
     return TRUE;
 }
 
+static gboolean
+menulist_toggle_menu (GtkToggleButton *button,
+                      GdkEventButton *ev, 
+                      Windowlist * wl)
+{
+    if (ev->button != 1)
+	return FALSE;
+    return menulist_popup_menu (wl, ev, FALSE);
+}
+
 /**
  * Check for urgent on workspaces
  **/
@@ -593,13 +708,9 @@ static gboolean
 windowlist_search_urgent (gpointer data)
 {
     Windowlist * wl = (Windowlist *) data;
-    
     NetkWindow *window;
-    NetkWindowState state;
     NetkWorkspace *active_workspace, *window_workspace;
-    
     gboolean blink = FALSE;
-    
     GList *windows, *li;
     
     windows = netk_screen_get_windows_stacked (wl->screen);
@@ -623,11 +734,7 @@ windowlist_search_urgent (gpointer data)
 	    netk_window_is_skip_tasklist (window))
 		continue;
 	
-	/* Get window state */
-	state = netk_window_get_state (window);
-	
-	/* Check if window is urgent */
-	if(G_UNLIKELY (state & NETK_WINDOW_STATE_URGENT))
+        if (netk_window_or_transient_demands_attention (window)) 
 	    blink = TRUE;
     }
     
@@ -790,6 +897,59 @@ windowlist_active_window_changed (GtkWidget * w,
     }
 }
 
+/**
+ * Handle user messages
+ **/
+static gboolean
+wl_message_received (GtkWidget *w, GdkEventClient *ev, gpointer user_data)
+{
+    Windowlist *wl = user_data;
+
+    if (ev->data_format == 8 && *(ev->data.b) != '\0')
+    {
+        if (!strcmp (XFCE_WINDOW_LIST_MESSAGE, ev->data.b))
+            return menulist_popup_menu (wl, NULL, FALSE);
+
+        if (!strcmp (XFCE_WINDOW_LIST_AT_POINTER_MESSAGE, ev->data.b))
+            return menulist_popup_menu (wl, NULL, TRUE);
+    }
+
+    return FALSE;
+}
+
+static gboolean
+wl_set_selection (Windowlist *wl)
+{
+    GdkScreen *gscreen;
+    gchar selection_name[32];
+    Atom selection_atom;
+    GtkWidget *win;
+    Window xwin;
+
+    win = gtk_invisible_new ();
+    gtk_widget_realize (win);
+    xwin = GDK_WINDOW_XID (GTK_WIDGET (win)->window);
+
+    gscreen = gtk_widget_get_screen (win);
+    g_snprintf (selection_name, sizeof (selection_name), 
+                XFCE_WINDOW_LIST_SELECTION"%d", gdk_screen_get_number (gscreen));
+    selection_atom = XInternAtom (GDK_DISPLAY (), selection_name, False);
+
+    if (XGetSelectionOwner (GDK_DISPLAY (), selection_atom))
+    {
+        gtk_widget_destroy (win);
+        return FALSE;
+    }
+
+    XSelectInput (GDK_DISPLAY (), xwin, PropertyChangeMask);
+    XSetSelectionOwner (GDK_DISPLAY (), selection_atom, xwin, GDK_CURRENT_TIME);
+
+    g_signal_connect (G_OBJECT (win), "client-event",
+                      G_CALLBACK (wl_message_received), wl);
+    
+    return TRUE;
+}
+
 /**
  * Build the panel button and connect signals and styles
  **/
@@ -819,7 +979,7 @@ windowlist_create_button (Windowlist * wl)
 					 GTK_ICON_SIZE_MENU, NULL);
 	    wl->icon = xfce_scaled_image_new_from_pixbuf (pb);
 	    gtk_container_add (GTK_CONTAINER (wl->button), wl->icon);
-	    g_object_unref (pb);
+	    g_object_unref (G_OBJECT (pb));
 	
 	    wl->screen_callback_id =
 		g_signal_connect (wl->screen, "active-window-changed",
@@ -857,7 +1017,9 @@ windowlist_create_button (Windowlist * wl)
     
     g_signal_connect (wl->button, "state-changed",
                       G_CALLBACK (windowlist_state_changed), wl);
-    
+
+    wl_set_selection (wl);
+
     gtk_widget_show_all (wl->button);
     gtk_container_add (GTK_CONTAINER (wl->plugin), wl->button);
     
@@ -1056,7 +1218,7 @@ windowlist_new (XfcePanelPlugin * plugin)
     GdkScreen *screen;
     int screen_idx;
     
-    Windowlist *wl = g_new0 (Windowlist, 1);
+    Windowlist *wl = panel_slice_new0 (Windowlist);
     
     /* Some default values if everything goes wrong */
     wl->layout			= DEF_BUTTON_LAYOUT;
@@ -1077,7 +1239,7 @@ windowlist_new (XfcePanelPlugin * plugin)
     wl->plugin = plugin;
     
     wl->tooltips = gtk_tooltips_new ();
-    g_object_ref (wl->tooltips);
+    g_object_ref (G_OBJECT (wl->tooltips));
     gtk_object_sink (GTK_OBJECT (wl->tooltips));
 
     /* get the screen where the widget is, for dual screen */
@@ -1129,6 +1291,8 @@ windowlist_set_size (XfcePanelPlugin *plugin,
 		    gtk_widget_set_size_request (GTK_WIDGET (wl->button),
 					 ARROW_WIDTH, size);
 		    break;
+                default:
+                    break;
 	    }
 	    break;
     }
@@ -1140,7 +1304,7 @@ static void
 windowlist_free (XfcePanelPlugin * plugin,
 		 Windowlist * wl)
 {
-    g_object_unref (wl->tooltips);
+    g_object_unref (G_OBJECT (wl->tooltips));
     
     if (wl->screen_callback_id)
 	g_signal_handler_disconnect (wl->screen, wl->screen_callback_id);
@@ -1165,7 +1329,7 @@ windowlist_free (XfcePanelPlugin * plugin,
     if (wl->button)
 	gtk_widget_destroy (wl->button);
     
-    g_free (wl);
+    panel_slice_free (Windowlist, wl);
 }
 
 static void
